/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.catalina.tribes.membership.cloud;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

public interface StreamProvider {
    /**
     * Open stream to the specified URL.
     *
     * @param url            the url
     * @param headers        the headers map
     * @param connectTimeout connection timeout in ms
     * @param readTimeout    read timeout in ms
     *
     * @return the stream
     *
     * @throws IOException when an error occurs
     */
    InputStream openStream(String url, Map<String,String> headers, int connectTimeout, int readTimeout)
            throws IOException;
}
